/*
	(C) 1995-99 AROS - The Amiga Research OS
	$Id: showwindow.c,v 1.4 2009/04/05 16:39:33 jacadcaps Exp $
 
	Desc: Intuition function ShowWindow()
	Lang: english
*/
#include <proto/layers.h>
#include "showhide.h"
#include "intuition_intern.h"
#include "intuition_private.h"
#include "inputhandler_actions.h"

#ifdef CGXSHOWHIDESUPPORT
#undef ChangeLayerVisibility
struct ShowWindowActionMsg
{
	struct IntuiActionMsg msg;
	struct Window *window;
};

static VOID int_showwindow(struct ShowWindowActionMsg *msg,
						   struct IntuitionBase *IntuitionBase);
#endif

#ifdef ChangeLayerVisibility
struct ShowWindowActionMsg
{
	struct IntuiActionMsg msg;
	struct Window *window;
};

static VOID int_showwindow(struct ShowWindowActionMsg *msg,
						   struct IntuitionBase *IntuitionBase);
#endif

/*****************************************************************************
 
	NAME */
#include <proto/intuition.h>

AROS_LH1(VOID, ShowWindow,

		 /*  SYNOPSIS */
		 AROS_LHA(struct Window *, window, A0),

		 /*  LOCATION */
		 struct IntuitionBase *, IntuitionBase, 140, Intuition)

/*  FUNCTION
		Make a window visible. This function does not bring the
		window back into the visible area of the screen but rather
		switches it into visible state.
 
 
	INPUTS
	window - The window to affect.
 
	RESULT
 
	NOTES
 
	EXAMPLE
 
	BUGS
 
	SEE ALSO
 
	INTERNALS
 
	HISTORY
 
*****************************************************************************/
{
	AROS_LIBFUNC_INIT
	AROS_LIBBASE_EXT_DECL(struct IntuitionBase *,IntuitionBase)

#ifdef CGXSHOWHIDESUPPORT
	struct ShowWindowActionMsg msg;

	DEBUG_SHOWWINDOW(dprintf("ShowWindow: Window 0x%lx\n", window));

	SANITY_CHECK(window)

	msg.window = window;
	DoASyncAction((APTR)int_showwindow, &msg.msg, sizeof(msg), IntuitionBase);
#endif

#ifdef ChangeLayerVisibility
	struct ShowWindowActionMsg msg;

	DEBUG_SHOWWINDOW(dprintf("ShowWindow: Window 0x%lx\n", window));

	msg.window = window;
	DoASyncAction((APTR)int_showwindow, &msg.msg, sizeof(msg), IntuitionBase);
#endif

	AROS_LIBFUNC_EXIT
} /* ShowWindow */

#ifdef CGXSHOWHIDESUPPORT
static VOID int_showwindow(struct ShowWindowActionMsg *msg,
						   struct IntuitionBase *IntuitionBase)
{
	struct Window *window = msg->window;

	SANITY_CHECK(window)

	if (((struct Library *)LayersBase)->lib_Version < 52)
	{
		if (!CGXSystemBase) return;
		CGXShowWindow(window);
	}
	else
	{
		LayersControlTags(&window->WScreen->LayerInfo,
						    LT_Layer,(ULONG)WLAYER(window),
						    LT_Opacity,IW(window)->useropacity,
						    (BLAYER(window) != WLAYER(window)) ? LT_Layer : TAG_IGNORE,(ULONG)BLAYER(window),
						    (BLAYER(window) != WLAYER(window)) ? LT_Opacity : TAG_IGNORE,IW(window)->useropacity,
						    TAG_DONE);
	}

	((struct IntWindow *)(window))->specialflags &= ~SPFLAG_ICONIFIED;
	ActivateWindow(window);
};
#endif

#ifdef ChangeLayerVisibility
static VOID int_showwindow(struct ShowWindowActionMsg *msg,
						   struct IntuitionBase *IntuitionBase)
{
	struct Window *window = msg->window;
	struct Screen *screen = window->WScreen;

	if (!IsWindowVisible(window))
	{
		struct Requester *req;

		LOCK_REFRESH(screen);

		if (BLAYER(window))
		{
			ChangeLayerVisibility(BLAYER(window), TRUE);
		}
		ChangeLayerVisibility(WLAYER(window), TRUE);

		for (req = window->FirstRequest; req; req = req->OlderRequest)
		{
			ChangeLayerVisibility(req->ReqLayer, TRUE);
		}

		CheckLayers(screen, IntuitionBase);

		UNLOCK_REFRESH(screen);
	}
}
#endif
